import os
from datetime import datetime
import pytz

from ems_config.vars import APP_DATA_DIR


def parse_dt_or_none(cfg_section, key, timezone=pytz.UTC, fmt="%Y-%m-%dT%H:%M:%SZ"):
    s = cfg_section.get(key, fallback=None)
    if s is None:
        return None
    d = datetime.strptime(s, fmt)
    if timezone:
        d = d.replace(tzinfo=pytz.UTC)
    return d


def parse_list_or_none(cfg_section, key, data_type=str, delimiter=','):
    s = cfg_section.get(key, None)
    if s is None:
        return None
    return [data_type(x.strip()) for x in s.split(delimiter) if x]


def get_from_app_data(p):
    return os.path.join(APP_DATA_DIR, p)


def get_from_cfg_or_env(cfg, cfg_key, key, default, prefix_cfg=True, env_key=None):
    env_key = env_key if env_key is not None else f"{cfg_key}_{key}" if prefix_cfg else key
    try:
        return cfg[cfg_key].get(key, fallback=os.getenv(env_key, default))
    except KeyError:
        return os.getenv(env_key, default)



import unittest
import tempfile
import os
from configparser import ConfigParser
from contextlib import contextmanager
from pathlib import Path
from shutil import copyfile
from unittest import mock

from ems_config import parse_config

res_path = Path(os.path.dirname(os.path.realpath(__file__))) / "res"


@contextmanager
def working_dir(w_dir=None):
    _original_dir = os.getcwd()
    temp_dir = None
    if w_dir is None:
        temp_dir = tempfile.TemporaryDirectory()
        w_dir = temp_dir.name

    os.chdir(w_dir)
    yield Path(w_dir)
    os.chdir(_original_dir)

    if temp_dir:
        temp_dir.cleanup()


class TestLoading(unittest.TestCase):

    def test_sanity(self):
        self.assertTrue(True)

    def test_load_ini_from_default(self):
        filename = "config.ini"
        with working_dir() as path:
            os.mkdir(path / "app_data")
            copyfile(res_path / filename, path / "app_data" / filename)

            c = parse_config()
            self.assertTrue(isinstance(c, ConfigParser))

            self.assertEqual(c["DEFAULT"]["field"], "value")

    def test_copy_and_load_ini_from_default(self):
        filename = "config.ini"
        with working_dir() as path:
            copyfile(res_path / filename, path / "config.example.ini")

            c = parse_config()
            self.assertTrue(isinstance(c, ConfigParser))

            self.assertEqual(c["DEFAULT"]["field"], "value")

    def test_load_ini_from_explicit_path(self):
        filename = "config.ini"
        with working_dir() as path:
            copyfile(res_path / filename, path / filename)

            c = parse_config(config_path=filename)
            self.assertTrue(isinstance(c, ConfigParser))

            self.assertEqual(c["DEFAULT"]["field"], "value")

    def test_load_json(self):
        filename = "config.yml"
        with working_dir() as path:
            copyfile(res_path / filename, path / filename)

            c = parse_config(filename)
            self.assertTrue(isinstance(c, dict))

            self.assertEqual(c["field"], "value")

    def test_load_yaml(self):
        filename = "config.yml"
        with working_dir() as path:
            copyfile(res_path / filename, path / filename)

            c = parse_config(filename)
            self.assertTrue(isinstance(c, dict))

            self.assertEqual(c["field"], "value")

    @mock.patch.dict(os.environ, {"CONFIG_PATH": "config.ini"})
    def test_copy_if_not_exists(self):
        filename = "config.ini"
        with working_dir():
            self.assertEqual(len(os.listdir()), 0)
            parse_config(str(res_path / filename))
            self.assertTrue(filename in os.listdir())

    def test_raise_if_unknown_ext(self):
        filename = "config.unknown"
        with working_dir():
            self.assertRaises(ValueError, lambda: parse_config(res_path / filename))



import io
import os
import requests

from pathlib import Path
from shutil import copyfile
from ems_config.vars import APP_DATA_DIR
from ems_config.helpers import get_from_app_data


def _read(f, path):
    # We let the user handle exceptions
    if path.endswith("yml"):
        import yaml
        return yaml.safe_load(f)
    if path.endswith("json"):
        import json
        return json.loads(f.read())
    if path.endswith("ini"):
        from configparser import ConfigParser
        cp = ConfigParser()
        cp.read_file(f, path)
        return cp
    raise ValueError(f"Unknown config extension: {path}")


def read_file(path):
    if isinstance(path, Path):
        path = str(path)
    with open(path, 'r') as f:
        return _read(f, path)


def parse_config(config_template=None, config_path=None):
    os.makedirs(APP_DATA_DIR, exist_ok=True)

    if config_template is None:
        if "CONFIG_TEMPLATE_PATH" in os.environ:
            config_template = os.getenv("CONFIG_TEMPLATE_PATH")
        else:
            config_template = "config.example.ini"

    if config_path is None:
        if "CONFIG_PATH" in os.environ:
            config_path = os.getenv("CONFIG_PATH")
        else:
            template_filename = os.path.basename(config_template)
            config_path = get_from_app_data(template_filename.replace(".example", ""))

    if not os.path.exists(config_path):
        dirname = os.path.dirname(config_path)
        if dirname:
            os.makedirs(dirname, exist_ok=True)

        copyfile(config_template, config_path)

    cfg = read_file(config_path)

    return cfg


def parse_remote_config(url="https://assets.ems.vestas.net/config.ini", verify=False):
    config_data = requests.get(url, verify=verify).text
    return _read(io.StringIO(config_data), url)
